@@ -196,7 +196,9 @@ function _M.init(env, args)
 
         local res, err
         local retry_time = 0
-        while retry_time < 2 do
+
+        local health_check_retry = tonumber(yaml_conf.etcd.health_check_retry) or 2
+        while retry_time < health_check_retry do
             res, err = request(version_url, yaml_conf)
             -- In case of failure, request returns nil followed by an error message.
             -- Else the first return value is the response body
