@@ -14,6 +14,11 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
+
+--- Get configuration information in Stand-alone mode.
+--
+-- @module core.config_yaml
+
 local config_local = require("apisix.core.config_local")
 local yaml         = require("tinyyaml")
 local log          = require("apisix.core.log")
@@ -22,6 +27,7 @@ local new_tab      = require("table.new")
 local check_schema = require("apisix.core.schema").check
 local profile      = require("apisix.core.profile")
 local lfs          = require("lfs")
+local file         = require("apisix.cli.file")
 local exiting      = ngx.worker.exiting
 local insert_tab   = table.insert
 local type         = type
@@ -100,6 +106,12 @@ local function read_apisix_yaml(premature, pre_mtime)
         return
     end
 
+    local ok, err = file.resolve_conf_var(apisix_yaml_new)
+    if not ok then
+        log.error("failed: failed to resolve variables:" .. err)
+        return
+    end
+
     apisix_yaml = apisix_yaml_new
     apisix_yaml_ctime = last_change_time
 end
