@@ -65,6 +65,18 @@ local type = type
 
 
 local events_list
+local exclude_resp_header = {
+    ["connection"] = true,
+    ["content-length"] = true,
+    ["transfer-encoding"] = true,
+    ["location"] = true,
+    ["server"] = true,
+    ["www-authenticate"] = true,
+    ["content-encoding"] = true,
+    ["content-type"] = true,
+    ["content-location"] = true,
+    ["content-language"] = true,
+}
 
 local function new_lrucache()
     return core.lrucache.new({
@@ -611,6 +623,17 @@ local rpc_handlers = {
                 end
             end
 
+            local len = rewrite:RespHeadersLength()
+            if len > 0 then
+                for i = 1, len do
+                    local entry = rewrite:RespHeaders(i)
+                    local name = entry:Name()
+                    if exclude_resp_header[str_lower(name)] == nil then
+                        core.response.set_header(name, entry:Value())
+                    end
+                end
+            end
+
             local len = rewrite:ArgsLength()
             if len > 0 then
                 local changed = {}
