@@ -20,6 +20,7 @@ local json              = core.json
 local pb                = require("pb")
 local ngx               = ngx
 local string            = string
+local ipairs            = ipairs
 local tonumber          = tonumber
 local type              = type
 
@@ -27,17 +28,27 @@ local type              = type
 local _M = {version = 0.1}
 
 
-function _M.find_method(protos, service, method)
-    local loaded = protos[proto_fake_file]
-    if not loaded or type(loaded) ~= "table" then
+function _M.find_method(proto, service, method)
+    local loaded = proto[proto_fake_file]
+    if type(loaded) ~= "table" then
+        core.log.error("compiled proto not found")
         return nil
     end
 
-    if not loaded.index[service] or type(loaded.index[service]) ~= "table" then
+    if type(loaded.index[service]) ~= "table" then
+        core.log.error("compiled proto service not found")
         return nil
     end
 
-    return loaded.index[service][method]
+    local res = loaded.index[service][method]
+    if not res then
+        core.log.error("compiled proto method not found")
+        return nil
+    end
+
+    -- restore pb state
+    pb.state(proto.pb_state)
+    return res
 end
 
 
@@ -100,6 +111,22 @@ function _M.map_message(field, default_values, request_table)
         if ty ~= "enum" and field_type:sub(1, 1) == "." then
             if request_table[name] == nil then
                 sub = default_values and default_values[name]
+            elseif core.table.isarray(request_table[name]) then
+                local sub_array = core.table.new(#request_table[name], 0)
+                for i, value in ipairs(request_table[name]) do
+                    local sub_array_obj
+                    if type(value) == "table" then
+                        sub_array_obj, err = _M.map_message(field_type,
+                                default_values and default_values[name], value)
+                        if err then
+                            return nil, err
+                        end
+                    else
+                        sub_array_obj = value
+                    end
+                    sub_array[i] = sub_array_obj
+                end
+                sub = sub_array
             else
                 sub, err = _M.map_message(field_type, default_values and default_values[name],
                                           request_table[name])
