@@ -171,12 +171,12 @@ end
 
 local function get_consumer(access_key)
     if not access_key then
-        return nil, {message = "missing access key"}
+        return nil, "missing access key"
     end
 
     local consumer_conf = consumer.plugin(plugin_name)
     if not consumer_conf then
-        return nil, {message = "Missing related consumer"}
+        return nil, "Missing related consumer"
     end
 
     local consumers = lrucache("consumers_key", consumer_conf.conf_version,
@@ -184,7 +184,7 @@ local function get_consumer(access_key)
 
     local consumer = consumers[access_key]
     if not consumer then
-        return nil, {message = "Invalid access key"}
+        return nil, "Invalid access key"
     end
     core.log.info("consumer: ", core.json.delay_encode(consumer))
 
@@ -244,7 +244,16 @@ local function generate_signature(ctx, secret_key, params)
 
             -- whether to encode the uri parameters
             if type(param) == "table" then
+                local vals = {}
                 for _, val in pairs(param) do
+                    if type(val) == "boolean" then
+                        val = ""
+                    end
+                    core.table.insert(vals, val)
+                end
+                core.table.sort(vals)
+
+                for _, val in pairs(vals) do
                     core.table.insert(query_tab, encode_or_not(key) .. "=" .. encode_or_not(val))
                 end
             else
@@ -288,11 +297,11 @@ end
 
 local function validate(ctx, params)
     if not params.access_key or not params.signature then
-        return nil, {message = "access key or signature missing"}
+        return nil, "access key or signature missing"
     end
 
     if not params.algorithm then
-        return nil, {message = "algorithm missing"}
+        return nil, "algorithm missing"
     end
 
     local consumer, err = get_consumer(params.access_key)
@@ -302,7 +311,7 @@ local function validate(ctx, params)
 
     local conf = consumer.auth_conf
     if conf.algorithm ~= params.algorithm then
-        return nil, {message = "algorithm " .. params.algorithm .. " not supported"}
+        return nil, "algorithm " .. params.algorithm .. " not supported"
     end
 
     core.log.info("clock_skew: ", conf.clock_skew)
@@ -310,13 +319,13 @@ local function validate(ctx, params)
         local time = ngx.parse_http_time(params.date)
         core.log.info("params.date: ", params.date, " time: ", time)
         if not time then
-            return nil, {message = "Invalid GMT format time"}
+            return nil, "Invalid GMT format time"
         end
 
         local diff = abs(ngx_time() - time)
         core.log.info("gmt diff: ", diff)
         if diff > conf.clock_skew then
-            return nil, {message = "Clock skew exceeded"}
+            return nil, "Clock skew exceeded"
         end
     end
 
@@ -326,7 +335,7 @@ local function validate(ctx, params)
         if params.signed_headers then
             for _, header in ipairs(params.signed_headers) do
                 if not headers_map[header] then
-                    return nil, {message = "Invalid signed header " .. header}
+                    return nil, "Invalid signed header " .. header
                 end
             end
         end
@@ -340,27 +349,27 @@ local function validate(ctx, params)
                   " generated_signature: ", generated_signature)
 
     if request_signature ~= generated_signature then
-        return nil, {message = "Invalid signature"}
+        return nil, "Invalid signature"
     end
 
     local validate_request_body = get_conf_field(params.access_key, "validate_request_body")
     if validate_request_body then
         local digest_header = params.body_digest
         if not digest_header then
-            return nil, {message = "Invalid digest"}
+            return nil, "Invalid digest"
         end
 
         local max_req_body = get_conf_field(params.access_key, "max_req_body")
         local req_body, err = core.request.get_body(max_req_body, ctx)
         if err then
-            return nil, {message = "Exceed body limit size"}
+            return nil, "Exceed body limit size"
         end
 
         req_body = req_body or ""
         local request_body_hash = ngx_encode_base64(
                 hmac_funcs[params.algorithm](secret_key, req_body))
         if request_body_hash ~= digest_header then
-            return nil, {message = "Invalid digest"}
+            return nil, "Invalid digest"
         end
     end
 
@@ -440,12 +449,9 @@ end
 function _M.rewrite(conf, ctx)
     local params = get_params(ctx)
     local validated_consumer, err = validate(ctx, params)
-    if err then
-        return 401, err
-    end
-
     if not validated_consumer then
-        return 401, {message = "Invalid signature"}
+        core.log.warn("client request can't be validated: ", err or "Invalid signature")
+        return 401, {message = "client request can't be validated"}
     end
 
     local consumer_conf = consumer.plugin(plugin_name)
