@@ -16,6 +16,7 @@
 --
 local limit_local_new = require("resty.limit.count").new
 local core = require("apisix.core")
+local apisix_plugin = require("apisix.plugin")
 local tab_insert = table.insert
 local ipairs = ipairs
 local pairs = pairs
@@ -108,35 +109,27 @@ local schema = {
         show_limit_quota_header = {type = "boolean", default = true}
     },
     required = {"count", "time_window"},
-    dependencies = {
-        policy = {
-            oneOf = {
-                {
-                    properties = {
-                        policy = {
-                            enum = {"local"},
-                        },
-                    },
+    ["if"] = {
+        properties = {
+            policy = {
+                enum = {"redis"},
+            },
+        },
+    },
+    ["then"] = policy_to_additional_properties.redis,
+    ["else"] = {
+        ["if"] = {
+            properties = {
+                policy = {
+                    enum = {"redis-cluster"},
                 },
-                core.table.merge({
-                    properties = {
-                        policy = {
-                            enum = {"redis"},
-                        },
-                    },
-                }, policy_to_additional_properties.redis),
-                core.table.merge({
-                    properties = {
-                        policy = {
-                            enum = {"redis-cluster"},
-                        },
-                    },
-                }, policy_to_additional_properties["redis-cluster"]),
-            }
-        }
+            },
+        },
+        ["then"] = policy_to_additional_properties["redis-cluster"],
     }
 }
 
+local schema_copy = core.table.deepcopy(schema)
 
 local _M = {
     version = 0.4,
@@ -159,7 +152,10 @@ function _M.check_schema(conf)
 
     if conf.group then
         local fields = {}
-        for k in pairs(schema.properties) do
+        -- When the goup field is configured,
+        -- we will use schema_copy to get the whitelist of properties,
+        -- so that we can avoid getting injected properties.
+        for k in pairs(schema_copy.properties) do
             tab_insert(fields, k)
         end
         local extra = policy_to_additional_properties[conf.policy]
@@ -230,7 +226,7 @@ function _M.access(conf, ctx)
     local key
     if conf.key_type == "var_combination" then
         local err, n_resolved
-        key, err, n_resolved = core.utils.resolve_var(conf_key, ctx.var);
+        key, err, n_resolved = core.utils.resolve_var(conf_key, ctx.var)
         if err then
             core.log.error("could not resolve vars in ", conf_key, " error: ", err)
         end
@@ -252,7 +248,10 @@ function _M.access(conf, ctx)
 
     -- here we add a separator ':' to mark the boundary of the prefix and the key itself
     if not conf.group then
-        key = ctx.conf_type .. ctx.conf_version .. ':' .. key
+        -- Here we use plugin-level conf version to prevent the counter from being resetting
+        -- because of the change elsewhere.
+        -- A route which reuses a previous route's ID will inherits its counter.
+        key = ctx.conf_type .. apisix_plugin.conf_version(conf) .. ':' .. key
     else
         key = conf.group .. ':' .. key
     end
