@@ -63,7 +63,7 @@ local attr_schema = {
 
 local _M = {
     version = 0.1,
-    priority = 11010,
+    priority = 12015,
     name = plugin_name,
     schema = schema
 }
@@ -211,9 +211,12 @@ end
 
 function _M.rewrite(conf, ctx)
     local headers = ngx.req.get_headers()
-    local uuid_val = get_request_id(conf.algorithm)
+    local uuid_val
     if not headers[conf.header_name] then
+        uuid_val = get_request_id(conf.algorithm)
         core.request.set_header(ctx, conf.header_name, uuid_val)
+    else
+        uuid_val = headers[conf.header_name]
     end
 
     if conf.include_in_response then
